# This is a quick hack :)
import os
import sys
import json
from os.path import join, isdir, isfile, basename, splitext
from mutagen.easyid3 import EasyID3
from mutagen.easymp4 import EasyMP4
from mutagen.oggvorbis import OggVorbis

supported_formats = [".mp3", ".m4a", ".ogg"]

def find_songs(root):
	songs = {"name": basename(root), "children": []}
	
	entries = os.listdir(root)
	files = [f for f in entries if isfile(join(root, f))]
	dirs = [d for d in entries if isdir(join(root, d))]
	for d in dirs:		
		songs["children"].append(find_songs(join(root, d)))

	# Sort the files. In case the metadata doesn't contain tracknumber, this is useful.
	files.sort()
	for index,f in enumerate(files):
		base, ext = splitext(f)
		if (ext not in supported_formats):
			continue
		url = join(root, f).encode("utf-8")
		title, artist, album, track, length, bitrate = parse_metadata(url, def_title=f, def_track=index+1)
		songs["children"].append({"name": title, "artist": artist, "album": album,
					  "tracknumber": track, "leaf": True, "length": length,
					  "bitrate": bitrate, "url": url})
			

	return songs

def parse_metadata(f, def_title="N/A", def_artist="N/A", def_album="N/A", def_track=1):
	try:
		if (f.endswith(".mp3")):
			m = EasyID3(f)
		elif (f.endswith(".m4a")):
			m = EasyMP4(f)
		elif (f.endswith(".ogg")):
			m = OggVorbis(f)
	except:
		return (def_title, def_artist, def_album)
	title = m.get("title", [def_title])[0]
	artist = m.get("artist", [def_artist])[0]
	album = m.get("album", [def_album])[0]
	# tracknumber is a string. We would like it as a number (because we sort by it).
	try:
		track = int(m.get("tracknumber", [def_track])[0])
	except ValueError:
		track = def_track
	# length is floating point number, but we'd rather have it as an int.
	length = int(m.info.length)
	bitrate = m.info.bitrate
	return (title.encode("utf-8"), artist.encode("utf-8"), album.encode("utf-8"), track, length, bitrate)

def create_json(path):
	return json.dumps(find_songs(path))

def main():
	path = sys.argv[1]
	print create_json(path)

if __name__ == "__main__":
    main()
