Ext.define('Music.view.music.Playlist', {
	extend: 'Ext.grid.Panel',
	alias: 'widget.playlist',
	store: 'Playlist',
	title: 'Now playing:',
	multiSelect: true,
	sortableColumns: false,
	viewConfig: {
		getRowClass: function(rec, idx, rowPrms, ds) {
			return rec.data.active ? 'music-playlist-active-row' : '';
		},
		plugins: {
			ptype: 'gridviewdragdrop',
			dragText: 'Drag song(s) to their new location in the playlist',
			ddGroup: 'playlistDrag'
		},
		listeners: {
			beforedrop: function(node, data, overModel, dropPosition) {
				// If copy is not allowed (that means we're dragging within the playlist itself)
				// just perform the default action.
				if (!data.copy) {
					return true;
				}
				var record = this.getRecord(node);
				if (record == undefined) {
					index = this.getStore().getCount();
				} else {
					var index = this.getStore().indexOf(record);
					if (dropPosition == 'after') {
						index++;
					}
				}
				
				// Insert copies of the records.
				for (record in data.records) {
					if (data.records[record].data.leaf)
					this.getStore().insert(index++, data.records[record].data);
				}
				// I should be able to return 0 here according to the docs but
				// it doesn't work in 4.0.2a! Hopefully this will be fixed in the future.
				return false;
			}
		},
		copy: false
	},
	tbar: [
		{xtype: 'button', text: 'Play', action: 'play'},
		{xtype: 'button', text: 'Pause', action: 'pause', enableToggle: true},
		{xtype: 'button', text: 'Prev', action: 'prev'},
		{xtype: 'button', text: 'Next', action: 'next'},
		{xtype: 'button', text: 'Remove from playlist', action: 'removeFromPlaylist'},
		{xtype: 'button', text: 'Clear playlist', action: 'clearPlaylist'},
		'-',
		{xtype: 'slider', itemId: 'progressSlider', decimalPrecision: 1, width: 250, useTips: false},
		' ',
		{xtype: 'tbtext', itemId: 'progressText', text: '0:00/0:00'},
		'->',
		{xtype: 'button', text: 'Google for Songmeanings', action: 'songmeanings'},
	],
	rbar: [
		'->',
		{xtype: 'slider', itemId: 'volumeSlider', height: 150, maxValue: 100, vertical: true},
		{xtype: 'image', src: 'static/resources/speaker.png', height: 24, width: 24}
	],
	columns: [
		{header: 'Name', dataIndex: 'name', flex: 1},
		{header: 'Artist', dataIndex: 'artist', flex: 1},
		{header: 'Album', dataIndex: 'album', flex: 1}
	],
	
	setNowPlaying: function(song) {
		this.setTitle('Now playing: ' + song);
	},
	updateProgress: function(currentTime, duration) {
		var slider = this.down('slider[itemId=progressSlider]');
		if (duration > 0) {
			if (!slider.thumbs[0].dragging) {
				// Update progress slider.
				var progress = currentTime / duration;			
				this.setProgressSlider(progress * 100);
				// Update progress text.
				this.setProgressText(currentTime, duration);
			}
		} else {
			this.setProgressSlider(0);
			this.setProgressText(0, 0);
		}
	},
	setVolumeSlider: function(value) {
		var slider = this.down('slider[itemId=volumeSlider]');
		slider.setValue(value);
	},
	setProgressSlider: function(value) {
		var slider = this.down('slider[itemId=progressSlider]');
		slider.setValue(value);
	},
	setProgressText: function(currentTime, duration) {
		if (!isNaN(currentTime) && !isNaN(duration)) {
			var currentTimeFmt = Diapason.utils.secondsToHms(currentTime);
			var durationFmt = Diapason.utils.secondsToHms(duration);
			var progress = this.down('tbtext[itemId=progressText]');
			progress.setText(currentTimeFmt + '/' + durationFmt);
		}
	}
});
